@@ -1,7 +1,9 @@
 package File::Tail;
 
 use strict;
-use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $Is_Win32);
+
+$Is_Win32 = ($^O =~ /win32/i) ? 1 : 0;
 
 require Exporter;
 
@@ -9,7 +11,7 @@ require Exporter;
 # Items to export into callers namespace by default. Note: do not export
 # names by default without a very good reason. Use EXPORT_OK instead.
 # Do not simply export all your public functions/methods/constants.
-$VERSION = '1.0';
+$VERSION = '1.3';
 
 
 # Preloaded methods go here.
@@ -260,6 +262,8 @@ sub new {
     $object->{"tail"}=($params{'tail'} || 0);
     $object->{"nowait"}=($params{'nowait'} || 0);
     $object->{"maxbuf"}=($params{'maxbuf'} || 16384);
+    warn "maxbuf should be big enough to hold at least one longest probable line, and preferably several\n"
+      unless $object->{"maxbuf"}>1024;
     $object->{"name_changes_callback"}=($params{'name_changes'} || undef);
     if (defined $params{'reset_tail'}) {
         $object->{"reset_tail"} = $params{'reset_tail'};
@@ -285,43 +289,44 @@ sub new {
 # Sets {tail} to value of {reset_tail}; effect is that first call 
 # uses {tail} and subsequent calls use {reset_tail}.
 sub position {
-    my $object=shift;
-    $object->{"endpos"}=sysseek($object->{handle},0,SEEK_END);
-    unless ($object->{"tail"}) {
-	$object->{endpos}=$object->{curpos}=
-	    sysseek($object->{handle},0,SEEK_END);
-    } elsif ($object->{"tail"}<0) {
-	$object->{endpos}=sysseek($object->{handle},0,SEEK_END);
-	$object->{curpos}=sysseek($object->{handle},0,SEEK_SET);
-    } else {
-	my $crs=0;
-	my $maxlen=sysseek($object->{handle},0,SEEK_END);
-	while ($crs<$object->{"tail"}+1) {
-	    my $avlen=length($object->{"buffer"})/($crs+1);
-	    $avlen=80 unless $avlen;
-	    my $calclen=$avlen*$object->{"tail"};
-	    $calclen+=1024 if $calclen<=length($object->{"buffer"});
-	    $calclen=$maxlen if $calclen>$maxlen;
-	    $object->{curpos}=sysseek($object->{handle},-$calclen,SEEK_END);
-	    sysread($object->{handle},$object->{"buffer"},
-		    $calclen);
-	    $object->{curpos}=sysseek($object->{handle},0,SEEK_CUR);
-	    $crs=$object->{"buffer"}=~tr/\n//;
-	    last if ($calclen>=$maxlen);
-	}
-	$object->{curpos}=sysseek($object->{handle},0,SEEK_CUR);
-	$object->{endpos}=sysseek($object->{handle},0,SEEK_END);
-	if ($crs>$object->{"tail"}) {
-	    my $toskip=$crs-$object->{"tail"};
-	    my $pos;
-	    $pos=index($object->{"buffer"},"\n");
-	    while (--$toskip) {
-		$pos=index($object->{"buffer"},"\n",$pos+1);
-	    }
-	    $object->{"buffer"}=substr($object->{"buffer"},$pos+1);
-	}
+  my $object=shift;
+  $object->{"endpos"}=sysseek($object->{handle},0,SEEK_END);
+  unless ($object->{"tail"}) {
+    $object->{endpos}=$object->{curpos}=
+      sysseek($object->{handle},0,SEEK_END);
+  } elsif ($object->{"tail"}<0) {
+    $object->{endpos}=sysseek($object->{handle},0,SEEK_END);
+    $object->{curpos}=sysseek($object->{handle},0,SEEK_SET);
+  } else {
+    my $crs=0;
+    my $maxlen=sysseek($object->{handle},0,SEEK_END);
+    while ($crs<$object->{"tail"}+1) {
+      my $avlen=length($object->{"buffer"})/($crs+1);
+      $avlen=80 unless $avlen;
+      my $calclen=$avlen*$object->{"tail"};
+      $calclen=length($object->{buffer})+1024 if int($calclen)<=length($object->{"buffer"});
+      $calclen=$maxlen if $calclen>$maxlen;
+      $object->{curpos}=sysseek($object->{handle},-$calclen,SEEK_END);
+      sysread($object->{handle},$object->{"buffer"},
+              $calclen);
+      $object->{"buffer"} =~ s/\015\012/\n/g if $Is_Win32;
+      $object->{curpos}=sysseek($object->{handle},0,SEEK_CUR);
+      $crs=$object->{"buffer"}=~tr/\n//;
+      last if ($calclen>=$maxlen);
     }
-    $object->{"tail"}=$object->{"reset_tail"};
+    $object->{curpos}=sysseek($object->{handle},0,SEEK_CUR);
+    $object->{endpos}=sysseek($object->{handle},0,SEEK_END);
+    if ($crs>$object->{"tail"}) {
+      my $toskip=$crs-$object->{"tail"};
+      my $pos;
+      $pos=index($object->{"buffer"},"\n");
+      while (--$toskip) {
+        $pos=index($object->{"buffer"},"\n",$pos+1);
+      }
+      $object->{"buffer"}=substr($object->{"buffer"},$pos+1);
+    }
+  }
+  $object->{"tail"}=$object->{"reset_tail"};
 }
 
 # Tries to open or reopen the file; failure is an error unless 
@@ -346,6 +351,7 @@ sub reset_pointers {
     my $newname;
     if ($oldhandle && $$object{'name_changes_callback'}) {
 	$newname=$$object{'name_changes_callback'}();
+        $object->{"input"}= $newname;
     } else {
 	$newname=$object->input;
     }
@@ -368,7 +374,7 @@ sub reset_pointers {
 	return;
     }
     binmode($newhandle);
-    
+
     if (defined($oldhandle)) {
 	# If file has not been changed since last OK read do not do anything
 	$st=stat($newhandle);
@@ -382,8 +388,8 @@ sub reset_pointers {
         # last read, and replaced (within that second) with a file of equal
         # length, we're out of luck. I don't see how to fix this.
 	if ($st->mtime<=int($object->{'lastread'})) {
-	    if ($st->size==$object->{"curpos"}) {
-		$object->{lastread} = $st->mtime; 
+	    if (($st->size==$object->{"curpos"}) && ($st->ino == $object->{"inode"})) {
+		$object->{lastread} = $st->mtime;
 		return;
 	    } else { 
 		# will continue further to reset
@@ -391,6 +397,7 @@ sub reset_pointers {
 	} else {
 	}
 	$object->{handle}=$newhandle;
+        $object->{inode} = $st->ino;
 	$object->position;
 	$object->{lastread} = $st->mtime;
 	close($oldhandle);
@@ -516,6 +523,8 @@ sub readin {
     while ($nlen>0) {
 	$len=sysread($object->{handle},$object->{"buffer"},
 		     $nlen,length($object->{"buffer"}));
+        $object->{"buffer"} =~ s/\015\012/\n/g if $Is_Win32;
+
 	last if $len==0; # Some busy filesystems return 0 sometimes, 
                              # and never give anything more from then on if 
                              # you don't give them time to rest. This return 
